/* $Id: nettest.c,v 1.1 1996/10/13 18:18:32 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            nettest.c
 *
 *  DESCRIPTION     Program for testing parts of the shhnet library.
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <netinet/in.h>

#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                       P R I V A T E    D A T A                         *
 *                                                                        *
 **************************************************************************/

static int sock;



/**************************************************************************
 *                                                                        *
 *                        P U B L I C    D A T A                          *
 *                                                                        *
 **************************************************************************/



/**************************************************************************
 *                                                                        *
 *                   P R I V A T E    F U N C T I O N S                   *
 *                                                                        *
 **************************************************************************/

static void
errorPrintf(const char *format, ...)
{
    va_list ap;

    printf("nettest shows you this error: ");
    va_start(ap, format);
    vfprintf(stdout, format, ap);
    va_end(ap);
}

static void
testIn(void)
{
    SNPort   port;
    SNAddr addr;
    char   ip[80], name[80], s[80];
    
    inHostNameToAddr("localhost", &addr);
    inHostAddrToStr(addr, ip);
    printf("inHostNameToAddr/inHostAddrToStr: `localhost' is %s\n", ip);
    inHostAddrToName(addr, name, sizeof(name));
    printf("inHostNameToAddr/inHostAddrToName: %s is `%s'\n", ip, name);
    printf("\n");

    inServiceToPort("echo", "tcp", &port);
    printf("inServiceToPort: echo/tcp is port %d\n", (int) ntohs(port));
    printf("\n");

    tcpClientOpen("localhost", "echo", &sock);
    inGetSockStr(sock, s, sizeof(s), 1);
    printf("inGetSockStr: localhost/echo open connection: `%s'\n", s);
    inGetPeerStr(sock, s, sizeof(s), 1);
    printf("inGetPeerStr: localhost/echo open connection: `%s'\n", s);
    tcpClose(sock);
    printf("\n");
}

static void
testTcpClient(void)
{
    char s[80];

    tcpClientOpen("localhost", "echo", &sock);
    printf("tcpPrintf: sending `abcdefg\\r\\n' to the echo server\n");
    tcpPrintf(sock, "abcdefg\r\n");
    tcpGets(sock, s, sizeof(s));
    printf("tcpGets: `%s'\n", s);
    printf("\n");

    printf("tcpWriteAll: sending `hello '\n");
    tcpWriteAll(sock, "hello ", 6);
    printf("tcpWriteAll: sending `there!\\r\\n'\n");
    tcpWriteAll(sock, "there!\r\n", 8);
    memset(s, '\0', sizeof(s));
    tcpReadAll(sock, s, 3);
    printf("tcpReadAll: got `%s' when reading 3 bytes\n", s);
    memset(s, '\0', sizeof(s));
    tcpReadAll(sock, s, 11);
    printf("tcpReadAll: got `%s' when reading 11 bytes\n", s);
    printf("\n");

    printf("tcpPuts: sending `hello\\r\\nthere\\r\\n'\n");
    tcpPuts(sock, "hello\r\nthere\r\n");
    tcpGets(sock, s, sizeof(s));
    snChopCrLf(s);
    printf("tcpGets/snChopCrLf: got `%s'\n", s);
    tcpGets(sock, s, sizeof(s));
    snChopCrLf(s);
    printf("tcpGets/snChopCrLf: got `%s'\n", s);
    printf("\n");

    tcpClose(sock);
}

static void
testError(void)
{
    SNErr err;
    SNPort  port;

    snSetErrorHandling(SN_RETURN_ERROR, NULL);
    printf("generating an error with no message from shhnet\n");
    err = inServiceToPort("qwe", "tcp", &port);
    printf("i got the error code %d (%s) in return\n",
	   (int) err, snErrToStr(err));
    printf("\n");

    snSetErrorHandling(SN_REPORT_ERROR_AND_RETURN, NULL);
    printf("generating an error with default message from shhnet\n");
    err = inServiceToPort("qwe", "tcp", &port);
    printf("i got the error code %d (%s) in return\n",
	   (int) err, snErrToStr(err));
    printf("\n");

    snSetErrorHandling(SN_REPORT_ERROR_AND_EXIT, errorPrintf);
    printf("generating an error with my message while shhnet aborts\n");
    err = inServiceToPort("qwe", "tcp", &port);
    printf("whee! I'm still here!\n");
}


/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

int
main(void)
{
    testIn();
    testTcpClient();
    testError(); /* aborts the program */
    return 0;
}
