/* $Id: servopen.c,v 0.7 1996/10/13 18:18:32 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            servopen.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     A function for opening server side connections
 *                  for both tcp and udp.
 *                  Used internally.
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          _snSrvOpenCommon
 *
 *  FUNCTION      Open a tcp- or udp-connection.
 *
 *  INPUT         type    The connection type: CONN_TCP or CONN_UDP
 *                service A service name, or a port number (ascii). If a
 *                        service name is given, it is mapped to a port
 *                        number.
 *
 *  OUTPUT        sock    she socket referencing this side of the
 *                        connection.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   The function will create a socket, and bind it to the
 *                given service/port.
 */
SNErr
_snSrvOpenCommon(SNConn type, const char *service, int *sock)
{
    SNErr err;
    SNPort  port;
    char  *servicename = NULL;

    switch (type) {
      case CONN_TCP:
	servicename = "tcp";
	break;
      case CONN_UDP:
	servicename = "udp";
	break;
    }
    
    /* get the port number corresponding to the given service. */
    if ((err = inServiceToPort(service, servicename, &port)) != SN_OK)
	return err;

    /* create a socket. */
    if ((err = _snCreateSocket(type, port, sock)) != SN_OK)
	return err;

    if (type == CONN_TCP)
	listen(*sock, 5);

    return SN_OK;
}
