/* $Id: shhmidr.h,v 1.2 1996/10/13 18:18:33 sverrehu Exp $ */
#ifndef SHHMIDR_H
#define SHHMIDR_H

/* Machine Independent Data Representation */
/* belongs to the shhnet library. */

#include <sys/types.h>  /* size_t */

#ifdef __cplusplus
  extern "C" {
#endif

/*
 *  Fundamental types. You may have to change these to match your system.
 */
typedef unsigned char BYTE;    /*  8 bits */
typedef unsigned short WORD;   /* 16 bits */
typedef unsigned long DWORD;   /* 32 bits */
typedef char SBYTE;            /*  8 bits, signed */
typedef short SWORD;           /* 16 bits, signed */
typedef long SDWORD;           /* 32 bits, signed */

#define midrSizeByte (sizeof(BYTE))
#define midrSizeWord (sizeof(WORD))
#define midrSizeDWord (sizeof(DWORD))
#define midrSizeSByte (sizeof(SBYTE))
#define midrSizeSWord (sizeof(SWORD))
#define midrSizeSDWord (sizeof(SDWORD))
#define midrSizeFixStr(x) ((x) * sizeof(char))

typedef struct {
    BYTE   *buff;     /* buffer to hold the data */
    size_t size;      /* size of the buffer */
    size_t len;       /* number of relevant bytes in the buffer */
    size_t idx;       /* position for reading/writing data */
} MIDR;

MIDR  *midrAlloc(size_t size);
void   midrFree(MIDR *midr);
size_t midrGetSize(const MIDR *midr);
BYTE  *midrGetBuff(const MIDR *midr);
void   midrSetLen(const MIDR *midr, size_t len);
size_t midrGetLen(const MIDR *midr);
void   midrRewind(MIDR *midr);
void   midrRewind(MIDR *midr);

void   midrPutByte(MIDR *midr, BYTE x);
BYTE   midrGetByte(MIDR *midr);
void   midrPutWord(MIDR *midr, WORD x);
WORD   midrGetWord(MIDR *midr);
void   midrPutDWord(MIDR *midr, DWORD x);
DWORD  midrGetDWord(MIDR *midr);

void   midrPutSByte(MIDR *midr, SBYTE x);
SBYTE  midrGetSByte(MIDR *midr);
void   midrPutSWord(MIDR *midr, SWORD x);
SWORD  midrGetSWord(MIDR *midr);
void   midrPutSDWord(MIDR *midr, SDWORD x);
SDWORD midrGetSDWord(MIDR *midr);

void   midrPutStr(MIDR *midr, const char *s);
char  *midrGetStr(MIDR *midr, char *s, size_t size);
void   midrPutByteBlock(const BYTE *b, size_t size, MIDR *midr);
BYTE  *midrGetByteBlock(MIDR *midr, BYTE *b, size_t size);

#ifdef __cplusplus
  }
#endif

#endif
