/* $Id: shhnet.h,v 0.7 1996/10/13 18:18:33 sverrehu Exp $ */
#ifndef SHHNET_H
#define SHHNET_H

/* belongs to the shhnet library. */

#include <sys/types.h>  /* size_t, fd_set */
#include <sys/time.h>   /* struct timeval */

#ifdef __cplusplus
  extern "C" {
#endif

typedef unsigned long  SNAddr;    /* an IP-address */
typedef unsigned short SNPort;    /* a port number */

typedef enum {
    SN_REPORT_ERROR_AND_EXIT,     /* display message, and abort program */
    SN_REPORT_ERROR_AND_RETURN,   /* display message, and return error code */
    SN_EXIT,                      /* be silent, and exit */
    SN_RETURN_ERROR               /* be silent, and return error code */
} SNErrorHandling;

typedef enum { 
    SN_OK = 0,                    /* no error */
    SN_ERR_ERRNO,                 /* error code found in errno */
    SN_ERR_H_ERRNO,               /* error code found in h_errno */
    SN_ERR_UNKNOWN_SERVICE        /* unknown service in inServiceToPort */
    /* remember to update snErrToStr() in error.c if adding new codes */
} SNErr;

void  snSetErrorHandling(SNErrorHandling type, void (*f)(const char *, ...));
const char *snErrToStr(SNErr err);

SNErr tcpClientOpen(const char *host, const char *service, int *sock);
SNErr tcpServerOpen(const char *service, int *sock);
SNErr tcpClose(int sock);
SNErr tcpAccept(int sock, int *newsock);
SNErr tcpRead(int sock, char *buf, size_t count, size_t *nread);
SNErr tcpWrite(int sock, const char *buf, size_t count, size_t *nwritten);
SNErr tcpReadAll(int sock, char *buf, size_t count);
SNErr tcpWriteAll(int sock, const char *buf, size_t count);
SNErr tcpGets(int sock, char *s, int n);
SNErr tcpPuts(int sock, const char *s);
SNErr tcpPrintf(int sock, const char *format, ...);
SNErr tcpLinger(int sock, int onoff);

SNErr udpClientOpen(const char *host, const char *service, int *sock);
SNErr udpServerOpen(const char *service, int *sock);
SNErr udpClientSock(int *sock);
SNErr udpClose(int sock);
SNErr udpRead(int sock, char *buf, size_t count, size_t *nread);
SNErr udpWrite(int sock, const char *buf, size_t count, size_t *nwritten);
SNErr udpReadFrom(int sock, char *buf, size_t count,
		  SNAddr *addr, SNPort *port, size_t *nread);
SNErr udpWriteTo(int sock, const char *buf, size_t count,
		 SNAddr addr, SNPort port, size_t *nwritten);

SNErr inHostAddrToStr(SNAddr addr, char *s);
SNErr inHostAddrToName(SNAddr addr, char *s, int len);
SNErr inHostNameToAddr(const char *s, SNAddr *addr);
SNErr inServiceToPort(const char *s, const char *proto, SNPort *port);
SNErr inGetSockAddrPort(int sock, SNAddr *addr, SNPort *port);
SNErr inGetSockStr(int sock, char *s, int len, int lookupName);
SNErr inGetPeerAddrPort(int sock, SNAddr *addr, SNPort *port);
SNErr inGetPeerStr(int sock, char *s, int len, int lookupName);

typedef void (*SNSigHandler)(int);
SNErr snClientSetupSigHandlers(SNSigHandler termHandler);
SNErr snServerSetupSigHandlers(SNSigHandler termHandler,
			       SNSigHandler hupHandler);

SNErr snSelect(int *n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
	       struct timeval *timeout);

int   snChopLf(char *s);
int   snChopCrLf(char *s);

#ifdef __cplusplus
  }
#endif

#endif
