/* $Id: tcpserv.c,v 0.4 1996/10/11 21:42:47 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            tcpserv.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Functions specific to the server side of
 *                  a tcp-connection.
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          tcpServerOpen
 *
 *  FUNCTION      Open a server side tcp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr tcpServerOpen(const char *service, int *sock);
 *
 *  INPUT         service a service name, or a port number (ascii). if a
 *                        service name is given, it is mapped to a port
 *                        number.
 *
 *  OUTPUT        sock    the socket referencing this side of the
 *                        connection.
 *
 *  DESCRIPTION   See _snSrvOpenCommon() in servopen.c
 */
SNErr
tcpServerOpen(const char *service, int *sock)
{
    return _snSrvOpenCommon(CONN_TCP, service, sock);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          tcpAccept
 *
 *  FUNCTION      Accept an incoming tcp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                int tcpAccept(int sock, int *newsock);
 *
 *  INPUT         sock    socket of tcp-connection.
 *
 *  OUTPUT        newsock the new socket.
 *
 *  RETURNS       Error code.
 */
SNErr
tcpAccept(int sock, int *newsock)
{
    int    clilen;
    struct sockaddr_in cli;

    clilen = sizeof(cli);
    /* Accept is one of the system calls on BSD4.3 that are not
     * automatically restarted when interrupted by signals, so we wrap
     * it in a little loop.  */
    do {
	*newsock = accept(sock, (struct sockaddr *) &cli, &clilen);
    } while (*newsock < 0 && errno == EINTR);
    if (*newsock < 0)
	return _snError(SN_ERR_ERRNO, "accept");
    return SN_OK;
}
